@@ -20,7 +20,7 @@
 #[macro_use]
 extern crate quick_error;
 
-use imagequant::*;
+use imagequant::{Image, QuantizationResult, Attributes};
 use imgref::*;
 use rgb::*;
 
@@ -200,10 +200,12 @@ impl Collector {
     ///
     /// If the first frame doesn't start at pts=0, the delay will be used for the last frame.
     pub fn add_frame_rgba(&mut self, frame_index: usize, image: ImgVec<RGBA8>, presentation_timestamp: f64) -> CatResult<()> {
+        debug_assert!(frame_index == 0 || presentation_timestamp > 0.);
         self.queue.push(frame_index, Ok((Self::resized_binary_alpha(image.into(), self.width, self.height)?, presentation_timestamp)))
     }
 
     pub(crate) fn add_frame_rgba_cow(&mut self, frame_index: usize, image: Img<Cow<[RGBA8]>>, presentation_timestamp: f64) -> CatResult<()> {
+        debug_assert!(frame_index == 0 || presentation_timestamp > 0.);
         self.queue.push(frame_index, Ok((Self::resized_binary_alpha(image, self.width, self.height)?, presentation_timestamp)))
     }
 
@@ -297,21 +299,22 @@ impl Writer {
     /// Avoids wasting palette on pixels identical to the background.
     ///
     /// `background` is the previous frame.
-    fn quantize(image: ImgRef<'_, RGBA8>, importance_map: &[u8], has_prev_frame: bool, settings: &Settings) -> CatResult<(Attributes, QuantizationResult, Image<'static>)> {
+    fn quantize(image: ImgVec<RGBA8>, importance_map: &[u8], has_prev_frame: bool, settings: &Settings) -> CatResult<(Attributes, QuantizationResult, Image<'static>)> {
         let mut liq = Attributes::new();
         if settings.fast {
-            liq.set_speed(10);
+            liq.set_speed(10)?;
         }
         let quality = if has_prev_frame {
             settings.color_quality().into()
         } else {
             100 // the first frame is too important to ruin it
         };
-        liq.set_quality(0, quality);
-        let mut img = liq.new_image_stride(image.buf(), image.width(), image.height(), image.stride(), 0.)?;
+        liq.set_quality(0, quality)?;
+        let (buf, width, height) = image.into_contiguous_buf();
+        let mut img = liq.new_image(buf, width, height, 0.)?;
         img.set_importance_map(importance_map)?;
         if has_prev_frame {
-            img.add_fixed_color(RGBA8::new(0, 0, 0, 0));
+            img.add_fixed_color(RGBA8::new(0, 0, 0, 0))?;
         }
         let res = liq.quantize(&mut img)?;
         Ok((liq, res, img))
@@ -319,10 +322,11 @@ impl Writer {
 
     fn remap(liq: Attributes, mut res: QuantizationResult, mut img: Image<'static>, background: Option<ImgRef<'_, RGBA8>>, settings: &Settings) -> CatResult<(ImgVec<u8>, Vec<RGBA8>)> {
         if let Some(bg) = background {
-            img.set_background(liq.new_image_stride(bg.buf(), bg.width(), bg.height(), bg.stride(), 0.)?)?;
+            let (buf, width, height) = bg.to_contiguous_buf();
+            img.set_background(liq.new_image(buf, width, height, 0.)?)?;
         }
 
-        res.set_dithering_level((settings.quality as f32 / 50.0 - 1.).max(0.));
+        res.set_dithering_level((settings.quality as f32 / 50.0 - 1.).max(0.))?;
 
         let (pal, pal_img) = res.remapped(&mut img)?;
         debug_assert_eq!(img.width() * img.height(), pal_img.len());
@@ -405,7 +409,7 @@ impl Writer {
 
     fn make_diffs(mut inputs: OrdQueueIter<DecodedImage>, quant_queue: Sender<DiffMessage>, settings: &Settings) -> CatResult<()> {
         let (first_frame, first_frame_pts) = inputs.next().transpose()?.ok_or(Error::NoFrames)?;
-        let mut prev_frame_pts = -1.0;
+        let mut prev_frame_pts = 0.;
 
         let mut denoiser = Denoiser::new(first_frame.width(), first_frame.height(), settings.quality);
 
@@ -460,6 +464,7 @@ impl Writer {
 
                 // conversion from pts to delay
                 let end_pts = if let Some((_, next_pts)) = next_frame {
+                    debug_assert!(next_pts > 0.);
                     next_pts - first_frame_pts
                 } else if first_frame_pts > 1. / 100. {
                     // this is gifski's weird rule that non-zero first-frame pts
@@ -467,7 +472,7 @@ impl Writer {
                     pts + first_frame_pts
                 } else {
                     // otherwise assume steady framerate
-                    pts + (pts - prev_frame_pts)
+                    (pts + (pts - prev_frame_pts)).max(1./100.)
                 };
                 debug_assert!(end_pts > 0.);
                 prev_frame_pts = pts;
@@ -518,7 +523,7 @@ impl Writer {
                     }
                 }
 
-                let (liq, remap, liq_image) = Self::quantize(image.as_ref(), &importance_map, ordinal_frame_number > 1, settings)?;
+                let (liq, remap, liq_image) = Self::quantize(image, &importance_map, ordinal_frame_number > 1, settings)?;
                 let max_loss = settings.gifsicle_loss();
                 for imp in &mut importance_map {
                     // encoding assumes rgba background looks like encoded background, which is not true for lossy
